import { EventEmitter, Directive, IterableDiffers, Input, Output, Component, Optional, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { __values } from 'tslib';
import { ReplaySubject } from 'rxjs';
import orderBy from 'lodash.orderby';
import min from 'lodash.min';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/angular-datatable.directive.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function SortEvent() { }
if (false) {
    /** @type {?} */
    SortEvent.prototype.sortBy;
    /** @type {?} */
    SortEvent.prototype.sortOrder;
}
/**
 * @record
 */
function PageEvent() { }
if (false) {
    /** @type {?} */
    PageEvent.prototype.activePage;
    /** @type {?} */
    PageEvent.prototype.rowsOnPage;
    /** @type {?} */
    PageEvent.prototype.dataLength;
}
/**
 * @record
 */
function DataEvent() { }
if (false) {
    /** @type {?} */
    DataEvent.prototype.length;
}
var AngularDatatableDirective = /** @class */ (function () {
    function AngularDatatableDirective(differs) {
        this.differs = differs;
        this.inputData = [];
        this.sortBy = "";
        this.sortOrder = "asc";
        this.sortByChange = new EventEmitter();
        this.sortOrderChange = new EventEmitter();
        this.rowsOnPage = 1000;
        this.activePage = 1;
        this.mustRecalculateData = false;
        this.onSortChange = new ReplaySubject(1);
        this.onPageChange = new EventEmitter();
        this.diff = differs.find([]).create(null);
    }
    /**
     * @return {?}
     */
    AngularDatatableDirective.prototype.getSort = /**
     * @return {?}
     */
    function () {
        return { sortBy: this.sortBy, sortOrder: this.sortOrder };
    };
    /**
     * @param {?} sortBy
     * @param {?} sortOrder
     * @return {?}
     */
    AngularDatatableDirective.prototype.setSort = /**
     * @param {?} sortBy
     * @param {?} sortOrder
     * @return {?}
     */
    function (sortBy, sortOrder) {
        if (this.sortBy !== sortBy || this.sortOrder !== sortOrder) {
            this.sortBy = sortBy;
            this.sortOrder = (/** @type {?} */ ((["asc", "desc"].includes(sortOrder) ? sortOrder : "asc")));
            this.mustRecalculateData = true;
            this.onSortChange.next({ sortBy: sortBy, sortOrder: (/** @type {?} */ (sortOrder)) });
            this.sortByChange.emit(this.sortBy);
            this.sortOrderChange.emit(this.sortOrder);
        }
    };
    /**
     * @return {?}
     */
    AngularDatatableDirective.prototype.getPage = /**
     * @return {?}
     */
    function () {
        return { activePage: this.activePage, rowsOnPage: this.rowsOnPage, dataLength: this.inputData.length };
    };
    /**
     * @param {?} activePage
     * @param {?} rowsOnPage
     * @return {?}
     */
    AngularDatatableDirective.prototype.setPage = /**
     * @param {?} activePage
     * @param {?} rowsOnPage
     * @return {?}
     */
    function (activePage, rowsOnPage) {
        if (this.rowsOnPage !== rowsOnPage || this.activePage !== activePage) {
            this.activePage = this.activePage !== activePage ? activePage : this.calculateNewActivePage(this.rowsOnPage, rowsOnPage);
            this.rowsOnPage = rowsOnPage;
            this.mustRecalculateData = true;
            this.onPageChange.emit({
                activePage: this.activePage,
                rowsOnPage: this.rowsOnPage,
                dataLength: this.inputData ? this.inputData.length : 0
            });
        }
    };
    /**
     * @private
     * @param {?} previousRowsOnPage
     * @param {?} currentRowsOnPage
     * @return {?}
     */
    AngularDatatableDirective.prototype.calculateNewActivePage = /**
     * @private
     * @param {?} previousRowsOnPage
     * @param {?} currentRowsOnPage
     * @return {?}
     */
    function (previousRowsOnPage, currentRowsOnPage) {
        /** @type {?} */
        var firstRowOnPage = (this.activePage - 1) * previousRowsOnPage + 1;
        /** @type {?} */
        var newActivePage = Math.ceil(firstRowOnPage / currentRowsOnPage);
        return newActivePage;
    };
    /**
     * @private
     * @return {?}
     */
    AngularDatatableDirective.prototype.recalculatePage = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var lastPage = Math.ceil(this.inputData.length / this.rowsOnPage);
        this.activePage = lastPage < this.activePage ? lastPage : this.activePage;
        this.activePage = this.activePage || 1;
        this.onPageChange.emit({
            activePage: this.activePage,
            rowsOnPage: this.rowsOnPage,
            dataLength: this.inputData.length
        });
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    AngularDatatableDirective.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes["rowsOnPage"]) {
            this.rowsOnPage = changes["rowsOnPage"].previousValue;
            this.setPage(this.activePage, changes["rowsOnPage"].currentValue);
            this.mustRecalculateData = true;
        }
        if (changes["sortBy"] || changes["sortOrder"]) {
            if (!["asc", "desc"].includes(this.sortOrder)) {
                console.warn("angular2-datatable: value for input mfSortOrder must be one of ['asc', 'desc'], but is:", this.sortOrder);
                this.sortOrder = "asc";
            }
            if (this.sortBy) {
                this.onSortChange.next({ sortBy: this.sortBy, sortOrder: this.sortOrder });
            }
            this.mustRecalculateData = true;
        }
        if (changes["inputData"]) {
            this.inputData = changes["inputData"].currentValue || [];
            this.recalculatePage();
            this.mustRecalculateData = true;
        }
    };
    /**
     * @return {?}
     */
    AngularDatatableDirective.prototype.ngDoCheck = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var changes = this.diff.diff(this.inputData);
        if (changes) {
            this.recalculatePage();
            this.mustRecalculateData = true;
        }
        if (this.mustRecalculateData) {
            this.fillData();
            this.mustRecalculateData = false;
        }
    };
    /**
     * @private
     * @return {?}
     */
    AngularDatatableDirective.prototype.fillData = /**
     * @private
     * @return {?}
     */
    function () {
        this.activePage = this.activePage;
        this.rowsOnPage = this.rowsOnPage;
        /** @type {?} */
        var offset = (this.activePage - 1) * this.rowsOnPage;
        /** @type {?} */
        var data = this.inputData;
        /** @type {?} */
        var sortBy = this.sortBy;
        if (typeof sortBy === 'string' || sortBy instanceof String) {
            data = orderBy(data, this.caseInsensitiveIteratee((/** @type {?} */ (sortBy))), [this.sortOrder]);
        }
        else {
            data = orderBy(data, sortBy, [this.sortOrder]);
        }
        data = data.slice(offset, offset + this.rowsOnPage);
        this.data = data;
    };
    /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    AngularDatatableDirective.prototype.caseInsensitiveIteratee = /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    function (sortBy) {
        return (/**
         * @param {?} row
         * @return {?}
         */
        function (row) {
            var e_1, _a;
            /** @type {?} */
            var value = row;
            try {
                for (var _b = __values(sortBy.split('.')), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var sortByProperty = _c.value;
                    if (value) {
                        value = value[sortByProperty];
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (value && typeof value === 'string' || value instanceof String) {
                return value.toLowerCase();
            }
            return value;
        });
    };
    AngularDatatableDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'table[mfData]',
                    exportAs: 'mfDataTable'
                },] }
    ];
    /** @nocollapse */
    AngularDatatableDirective.ctorParameters = function () { return [
        { type: IterableDiffers }
    ]; };
    AngularDatatableDirective.propDecorators = {
        inputData: [{ type: Input, args: ["mfData",] }],
        sortBy: [{ type: Input, args: ["mfSortBy",] }],
        sortOrder: [{ type: Input, args: ["mfSortOrder",] }],
        sortByChange: [{ type: Output, args: ["mfSortByChange",] }],
        sortOrderChange: [{ type: Output, args: ["mfSortOrderChange",] }],
        rowsOnPage: [{ type: Input, args: ["mfRowsOnPage",] }],
        activePage: [{ type: Input, args: ["mfActivePage",] }]
    };
    return AngularDatatableDirective;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    AngularDatatableDirective.prototype.diff;
    /** @type {?} */
    AngularDatatableDirective.prototype.inputData;
    /** @type {?} */
    AngularDatatableDirective.prototype.sortBy;
    /** @type {?} */
    AngularDatatableDirective.prototype.sortOrder;
    /** @type {?} */
    AngularDatatableDirective.prototype.sortByChange;
    /** @type {?} */
    AngularDatatableDirective.prototype.sortOrderChange;
    /** @type {?} */
    AngularDatatableDirective.prototype.rowsOnPage;
    /** @type {?} */
    AngularDatatableDirective.prototype.activePage;
    /**
     * @type {?}
     * @private
     */
    AngularDatatableDirective.prototype.mustRecalculateData;
    /** @type {?} */
    AngularDatatableDirective.prototype.data;
    /** @type {?} */
    AngularDatatableDirective.prototype.onSortChange;
    /** @type {?} */
    AngularDatatableDirective.prototype.onPageChange;
    /**
     * @type {?}
     * @private
     */
    AngularDatatableDirective.prototype.differs;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/default-sorter.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DefaultSorterComponent = /** @class */ (function () {
    function DefaultSorterComponent(mfTable) {
        this.mfTable = mfTable;
        this.isSortedByMeAsc = false;
        this.isSortedByMeDesc = false;
    }
    /**
     * @return {?}
     */
    DefaultSorterComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.mfTable.onSortChange.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this.isSortedByMeAsc = (event.sortBy == _this.sortBy && event.sortOrder == "asc");
            _this.isSortedByMeDesc = (event.sortBy == _this.sortBy && event.sortOrder == "desc");
        }));
    };
    /**
     * @return {?}
     */
    DefaultSorterComponent.prototype.sort = /**
     * @return {?}
     */
    function () {
        if (this.isSortedByMeAsc) {
            this.mfTable.setSort(this.sortBy, "desc");
        }
        else {
            this.mfTable.setSort(this.sortBy, "asc");
        }
    };
    DefaultSorterComponent.decorators = [
        { type: Component, args: [{
                    selector: 'mf-default-sorter',
                    template: "\n  <a style=\"cursor: pointer\" (click)=\"sort()\" class=\"text-nowrap\">\n      <ng-content></ng-content>\n      <span *ngIf=\"isSortedByMeAsc\" class=\"glyphicon glyphicon-triangle-top fa fa-sort-up\" aria-hidden=\"true\"></span>\n      <span *ngIf=\"isSortedByMeDesc\" class=\"glyphicon glyphicon-triangle-bottom fa fa-sort-down\" aria-hidden=\"true\"></span>\n  </a>"
                }] }
    ];
    /** @nocollapse */
    DefaultSorterComponent.ctorParameters = function () { return [
        { type: AngularDatatableDirective }
    ]; };
    DefaultSorterComponent.propDecorators = {
        sortBy: [{ type: Input, args: ["by",] }]
    };
    return DefaultSorterComponent;
}());
if (false) {
    /** @type {?} */
    DefaultSorterComponent.prototype.sortBy;
    /** @type {?} */
    DefaultSorterComponent.prototype.isSortedByMeAsc;
    /** @type {?} */
    DefaultSorterComponent.prototype.isSortedByMeDesc;
    /**
     * @type {?}
     * @private
     */
    DefaultSorterComponent.prototype.mfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/paginator.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PaginatorComponent = /** @class */ (function () {
    function PaginatorComponent(injectMfTable) {
        var _this = this;
        this.injectMfTable = injectMfTable;
        this.dataLength = 0;
        this.onPageChangeSubscriber = (/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this.activePage = event.activePage;
            _this.rowsOnPage = event.rowsOnPage;
            _this.dataLength = event.dataLength;
            _this.lastPage = Math.ceil(_this.dataLength / _this.rowsOnPage);
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    PaginatorComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        this.mfTable = this.inputMfTable || this.injectMfTable;
        this.onPageChangeSubscriber(this.mfTable.getPage());
        this.mfTable.onPageChange.subscribe(this.onPageChangeSubscriber);
    };
    /**
     * @param {?} pageNumber
     * @return {?}
     */
    PaginatorComponent.prototype.setPage = /**
     * @param {?} pageNumber
     * @return {?}
     */
    function (pageNumber) {
        this.mfTable.setPage(pageNumber, this.rowsOnPage);
    };
    /**
     * @param {?} rowsOnPage
     * @return {?}
     */
    PaginatorComponent.prototype.setRowsOnPage = /**
     * @param {?} rowsOnPage
     * @return {?}
     */
    function (rowsOnPage) {
        this.mfTable.setPage(this.activePage, rowsOnPage);
    };
    PaginatorComponent.decorators = [
        { type: Component, args: [{
                    selector: 'mf-paginator',
                    template: "<ng-content></ng-content>"
                }] }
    ];
    /** @nocollapse */
    PaginatorComponent.ctorParameters = function () { return [
        { type: AngularDatatableDirective, decorators: [{ type: Optional }] }
    ]; };
    PaginatorComponent.propDecorators = {
        inputMfTable: [{ type: Input, args: ["mfTable",] }]
    };
    return PaginatorComponent;
}());
if (false) {
    /** @type {?} */
    PaginatorComponent.prototype.inputMfTable;
    /**
     * @type {?}
     * @private
     */
    PaginatorComponent.prototype.mfTable;
    /** @type {?} */
    PaginatorComponent.prototype.activePage;
    /** @type {?} */
    PaginatorComponent.prototype.rowsOnPage;
    /** @type {?} */
    PaginatorComponent.prototype.dataLength;
    /** @type {?} */
    PaginatorComponent.prototype.lastPage;
    /**
     * @type {?}
     * @private
     */
    PaginatorComponent.prototype.onPageChangeSubscriber;
    /**
     * @type {?}
     * @private
     */
    PaginatorComponent.prototype.injectMfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/bootstrap-paginator.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BootstrapPaginatorComponent = /** @class */ (function () {
    function BootstrapPaginatorComponent() {
        this.rowsOnPageSet = [];
        this.minRowsOnPage = 0;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    BootstrapPaginatorComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.rowsOnPageSet) {
            this.minRowsOnPage = min(this.rowsOnPageSet);
        }
    };
    BootstrapPaginatorComponent.decorators = [
        { type: Component, args: [{
                    selector: 'mf-bootstrap-paginator',
                    template: "\n    <mf-paginator #p [mfTable]=\"mfTable\">\n        <ul class=\"pagination\" *ngIf=\"p.dataLength > p.rowsOnPage\">\n            <li class=\"page-item\" [class.disabled]=\"p.activePage <= 1\" (click)=\"p.setPage(1)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">&laquo;</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 4 && p.activePage + 1 > p.lastPage\" (click)=\"p.setPage(p.activePage - 4)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-4}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 3 && p.activePage + 2 > p.lastPage\" (click)=\"p.setPage(p.activePage - 3)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-3}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 2\" (click)=\"p.setPage(p.activePage - 2)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-2}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 1\" (click)=\"p.setPage(p.activePage - 1)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-1}}</a>\n            </li>\n            <li class=\"page-item active\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 1 <= p.lastPage\" (click)=\"p.setPage(p.activePage + 1)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+1}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 2 <= p.lastPage\" (click)=\"p.setPage(p.activePage + 2)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+2}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 3 <= p.lastPage && p.activePage < 3\" (click)=\"p.setPage(p.activePage + 3)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+3}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 4 <= p.lastPage && p.activePage < 2\" (click)=\"p.setPage(p.activePage + 4)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+4}}</a>\n            </li>\n            <li class=\"page-item\" [class.disabled]=\"p.activePage >= p.lastPage\" (click)=\"p.setPage(p.lastPage)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">&raquo;</a>\n            </li>\n        </ul>\n        <ul class=\"pagination pull-right float-sm-right\" *ngIf=\"p.dataLength > minRowsOnPage\">\n            <li class=\"page-item\" *ngFor=\"let rows of rowsOnPageSet\" [class.active]=\"p.rowsOnPage===rows\" (click)=\"p.setRowsOnPage(rows)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{rows}}</a>\n            </li>\n        </ul>\n    </mf-paginator>\n    "
                }] }
    ];
    BootstrapPaginatorComponent.propDecorators = {
        rowsOnPageSet: [{ type: Input, args: ["rowsOnPageSet",] }],
        mfTable: [{ type: Input, args: ["mfTable",] }]
    };
    return BootstrapPaginatorComponent;
}());
if (false) {
    /** @type {?} */
    BootstrapPaginatorComponent.prototype.rowsOnPageSet;
    /** @type {?} */
    BootstrapPaginatorComponent.prototype.mfTable;
    /** @type {?} */
    BootstrapPaginatorComponent.prototype.minRowsOnPage;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/angular-datatable.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AngularDatatableModule = /** @class */ (function () {
    function AngularDatatableModule() {
    }
    AngularDatatableModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: [AngularDatatableDirective, DefaultSorterComponent, PaginatorComponent, BootstrapPaginatorComponent],
                    exports: [AngularDatatableDirective, DefaultSorterComponent, PaginatorComponent, BootstrapPaginatorComponent]
                },] }
    ];
    return AngularDatatableModule;
}());

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: troplabs-angular-datatable.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AngularDatatableDirective, AngularDatatableModule, BootstrapPaginatorComponent, DefaultSorterComponent, PaginatorComponent };
//# sourceMappingURL=troplabs-angular-datatable.js.map
